@@ -1,9 +1,12 @@
 from typing import Union
+from copy import deepcopy
+from functools import reduce
 
 import numpy as np
 import pandas as pd
 import altair as alt
 from sklearn.metrics import pairwise_distances
+from sklearn.metrics.pairwise import paired_distances
 
 from whatlies.embedding import Embedding
 from whatlies.common import plot_graph_layout
@@ -39,6 +42,47 @@ class EmbeddingSet:
             # we assume it is a tuple of tokens
             self.embeddings = {t.name: t for t in embeddings}
 
+    def __contains__(self, item):
+        """
+        Checks if an item is in the embeddingset.
+
+        Usage:
+
+        ```python
+        from whatlies.embedding import Embedding
+        from whatlies.embeddingset import EmbeddingSet
+
+        foo = Embedding("foo", [0.1, 0.3])
+        bar = Embedding("bar", [0.7, 0.2])
+        buz = Embedding("buz", [0.1, 0.9])
+        emb = EmbeddingSet(foo, bar)
+
+        "foo" in emb # True
+        "dinosaur" in emb # False
+        ```
+        """
+        return item in self.embeddings.keys()
+
+    def __iter__(self):
+        """
+        Iterate over all the embeddings in the embeddingset.
+
+        Usage:
+
+        ```python
+        from whatlies.embedding import Embedding
+        from whatlies.embeddingset import EmbeddingSet
+
+        foo = Embedding("foo", [0.1, 0.3])
+        bar = Embedding("bar", [0.7, 0.2])
+        buz = Embedding("buz", [0.1, 0.9])
+        emb = EmbeddingSet(foo, bar)
+
+        [e for e in emb]
+        ```
+        """
+        return self.embeddings.values().__iter__()
+
     def __add__(self, other):
         """
         Adds an embedding to each element in the embeddingset.
@@ -340,6 +384,30 @@ class EmbeddingSet:
         by_similarity = sorted(zip(queries, distances), key=lambda z: z[1])
         return [(self[q], float(d)) for q, d in by_similarity[:n]]
 
+    def to_matrix(self):
+        return np.array([w.vector for w in self.embeddings.values()])
+
+    def movement_df(self, other, metric="euclidean"):
+        overlap = list(set(self.embeddings.keys()).union(set(other.embeddings.keys())))
+        mat1 = np.array([w.vector for w in self[overlap]])
+        mat2 = np.array([w.vector for w in other[overlap]])
+        return pd.DataFrame({
+            'name': overlap,
+            'movement': paired_distances(mat1, mat2, metric)
+        }).sort_values(['movement'], ascending=False).reset_index()
+
+    def to_axis_df(self, x_axis, y_axis):
+        if isinstance(x_axis, str):
+            x_axis = self[x_axis]
+        if isinstance(y_axis, str):
+            y_axis = self[y_axis]
+        return pd.DataFrame({
+            "x_axis": self.compare_against(x_axis),
+            "y_axis": self.compare_against(y_axis),
+            "name": [v.name for v in self.embeddings.values()],
+            "original": [v.orig for v in self.embeddings.values()],
+        })
+
     def plot(
         self,
         kind: str = "scatter",
@@ -374,6 +442,70 @@ class EmbeddingSet:
         plot_graph_layout(self.embeddings, kind, **kwargs)
         return self
 
+    def plot_difference(self, other,
+        x_axis: Union[str, Embedding],
+        y_axis: Union[str, Embedding],
+        annot: bool = True,
+    ):
+        """
+        Makes highly interactive plot of the difference between two sets of embeddings.
+
+        Arguments:
+            other: the other embeddingset
+            x_axis: the x-axis to be used, must be given when dim > 2
+            y_axis: the y-axis to be used, must be given when dim > 2
+            annot: drawn points should be annotated
+
+        **Usage**
+
+        ```python
+        from whatlies.language import SpacyLanguage
+
+        words = ["prince", "princess", "nurse", "doctor", "banker", "man", "woman",
+                 "cousin", "neice", "king", "queen", "dude", "guy", "gal", "fire",
+                 "dog", "cat", "mouse", "red", "bluee", "green", "yellow", "water",
+                 "person", "family", "brother", "sister"]
+
+        lang = SpacyLanguage("en_core_web_md")
+        emb = lang[words]
+        emb_new = emb - emb['king']
+
+        emb.plot_difference(emb_new, 'man', 'woman')
+        ```
+        """
+        if isinstance(x_axis, str):
+            x_axis = self[x_axis]
+        if isinstance(y_axis, str):
+            y_axis = self[y_axis]
+
+        df1 = (self.to_axis_df(x_axis, y_axis)
+               .set_index('original')
+               .drop(columns=['name']))
+        df2 = (other.to_axis_df(x_axis, y_axis)
+               .set_index('original')
+               .drop(columns=['name'])
+               .loc[lambda d: d.index.isin(df1.index)])
+        df_draw = (pd.concat([df1, df2])
+                   .reset_index()
+                   .sort_values(['original'])
+                   .assign(constant=1))
+
+        plots = []
+        for idx, grp_df in df_draw.groupby('original'):
+            _ = (alt.Chart(grp_df)
+                 .mark_line(color="gray", strokeDash=[2, 1])
+                 .encode(x='x_axis:Q', y='y_axis:Q'))
+            plots.append(_)
+        p0 = reduce(lambda x, y: x + y, plots)
+
+        p1 = (deepcopy(self)
+              .add_property("group", lambda d: "before")
+              .plot_interactive(x_axis, y_axis, annot=annot, show_axis_point=True, color="group"))
+        p2 = (deepcopy(other)
+              .add_property("group", lambda d: "after")
+              .plot_interactive(x_axis, y_axis, annot=annot, show_axis_point=True, color="group"))
+        return p0 + p1 + p2
+
     def plot_interactive(
         self,
         x_axis: Union[str, Embedding],
